global g_achievements;

function initAchievementDefinitions()
{
	//"origin" must always be "origin=" because we only support checking for equality
	g_achievements= [
		//f9
		{"format=":9,"origin=":"D","numparts>=":7},
		{"format=":9,"origin=":"D","vertpos>":0.5},
		{"format=":9,"origin=":"E","vertpos>":0.5}, //show evolution for the same criteria as previously design
		{"format=":9,"origin=":"D","vertpos>":0.5,"numjoints<=":7},
		{"format=":9,"vertpos>":1.5,"numparts<":8},
		//f1
		{"format=":1,"origin=":"D","numparts>=":8},
		{"format=":1,"numparts<":10,"vertpos>":1.5},
		{"format=":1,"origin=":"E","vertpos>":2}, //last vertpos requires evolution because the following goals (velocity) concern evolution...
		//any format
		{"numparts>":3,"velocity>":0.1},
		{"velocity>":0.2},
		{"numparts>":7,"velocity>":0.2},
		{"water_velocity>":0.01}
	];
}

function getCriterionAndRelation(crit)
{
	var rel="";
	var RELS="<=>";
	while(String.indexOf(RELS,String.right(crit,1))>=0)
	{
		rel=String.right(crit,1)+rel;
		crit=String.left(crit,String.len(crit)-1);
	}
	if (String.len(rel)<1)
		Simulator.message("Invalid relation in Achievement definition!",2);
	return [crit,rel];
}

function getPropertyValue(genotype,propname)
{
	if (propname=="format") //TODO really, format is the only field in Geno not available in Genotype? (...and we wanted these classes to become one since long time ago?)
		return genotype.geno.format-String.code("0");
	else if (propname=="origin")
		{
		if (genotype.data->edited) return "D";
		if (genotype.data->evaluations) return "E";
		return "?"; //other
		}
        else if (propname=="water_velocity")
                {
		var v=genotype.data[propname];
		if (v==null) v=0.0;
		return v;
		}
	else
		return genotype.[propname];
}

function quote(str)
{
	return "\""+str+"\"";
}

function checkAchievement(achievement,genotype)
{
	var ok=1;
	for(var i=0;i<achievement.size;i++)
	{
		var crit=getCriterionAndRelation(achievement.getKey(i));
		var actual=getPropertyValue(genotype,crit[0]);
		var required=achievement.get(i);
		if (crit[0]=="origin") //add "" around values because we know they are strings
		{
			actual=quote(actual);
			required=quote(required);
		}
		var rel=crit[1];
		if (rel=="=")
			rel="==";
		var test="return "+actual+rel+required+";";
		//CLI.println(test);
		var testresult=Simulator.eval(test);
		if (typeof(testresult)=="EvalError")
		{
			Simulator.message("Invalid expression in Achievement evaluation: "+test,2);
			Simulator.message(testresult.message,2);
			testresult=0;
		}
		ok=ok && testresult;
	}
	return ok;
}

function calcCompletedAchievements()
{
var count=0;
for(var q in g_achievements)
	for(var g in GenePools[0])
		if (checkAchievement(q,g)==1)
			   {count++; break;}
return count;
}
